@@ -10,9 +10,8 @@ Thing->columns(Essential => qw(id title description date));
 sub class_title { 'Thing' }
 sub class_plural { 'Things' }
 sub class_description { 'Things of one kind or another.' }
-sub class_precedes { undef }
 
-sub factory { return Class::DBI::Factory->instance('_test'); }
+sub factory { return Class::DBI::Factory->instance; }
 sub db_Main { return shift->factory->dbh; }
 
 1;
